
/*
	API Browser
	1.0.0
	by Brent P. Newhall <brent@other-space.com>
*/

#ifndef _BROWSER_H
#define _BROWSER_H

#include <vector>

#include <util/application.h>
#include <util/string.h>
#include <gui/window.h>
#include <gui/view.h>
#include <gui/rect.h>
#include <gui/treeview.h>
#include <gui/textview.h>
#include <gui/splitter.h>
#include <gui/menu.h>
#include <storage/directory.h>
#include <storage/file.h>

#include <codeview/codeview.h>
#include <codeview/format_c.h>

#include "statusbar.h"
#include "finddialog.h"

const unsigned int MSG_FILE_SELECTED = 1001;

class APIBrowserWindow : public os::Window
{
public:
	APIBrowserWindow( os::Window * pcParent, const os::Rect & r ); /* Constructor */
	virtual void HandleMessage( os::Message * pcMessage );
	virtual bool OkToQuit();

	void PopulateTree( const os::String path, const int indent );
	void SetupStatusBar();
	void SetupMenus();
	void Find( const os::String &pcString, bool bDown, bool bCaseSensitive, bool bExtended );
	void UpdateStatus( const os::String & );

private:
	os::Splitter * splitter;
	os::TreeView *treeView;
	os::Menu *menu;
	os::Window *pcParentWindow;
	os::String cSearchText;

	cv::CodeView *pcEditor;
	StatusBar *statusbar;
	FindDialog *findDialog;
	std::vector < os::String >filenames;

	bool bSearchDown;
	bool bSearchCase;
	bool bSearchExtended;
};

#endif //BROWSER_H
