//  Sourcery -:-  (C)opyright 2003-2004 Rick Caudill
//
// This is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include <storage/filereference.h>
#include <unistd.h>

#include "convertdialog.h"
#include "commonfuncs.h"

/**********************************************
* Description: ConvertDialog constructor
* Author: Rick Caudill
* Date: Thu Mar 18 22:57:43 2004
**********************************************/
ConvertDialog::ConvertDialog( const os::String &cFormat, const os::String &cFile ) : os::Window( os::Rect( 0, 0, 220, 100 ), "convert", "Convert to html:", os::WND_NO_ZOOM_BUT | os::WND_NO_DEPTH_BUT | os::WND_NOT_RESIZABLE )
{
	cMainFormat = cFormat;
	cFileName = cFile;
	Layout();
	SetTabs();
	pcFileReq = new os::FileRequester( os::FileRequester::LOAD_REQ, new os::Messenger( this ), getenv( "$HOME" ), os::FileRequester::NODE_FILE );
}

/**********************************************
* Description: ConvertDialog destructor
* Author: Rick Caudill
* Date: Thu Mar 18 22:57:43 2004
**********************************************/
ConvertDialog::~ConvertDialog()
{
}

/**********************************************
* Description: Handles messages between the convert dialog
* Author: Rick Caudill
* Date: Thu Mar 18 22:57:43 2004
**********************************************/
void ConvertDialog::HandleMessage( os::Message * pcMessage )
{
	switch ( pcMessage->GetCode() )
	{
		case M_CONVERT_CANCEL:
		{
			OkToQuit();
			break;
		}
		case M_CONVERT:
		{
			Convert();
			break;
		}
		case M_REQ_OPEN:
		{
			Lock();
			pcFileReq->CenterInWindow( this );
			pcFileReq->Show();
			pcFileReq->MakeFocus();
			//fall thru to os::M_LOAD_REQUESTED
		}
		case os::M_LOAD_REQUESTED:
		{
			os::String cPath;
			Unlock();
			if( pcMessage->FindString( "file/path", &cPath ) == 0 )
			{
				if( cPath[cPath.size() - 1] != '/' )
					cPath += "/";
				pcTVSaveTo->Set( cPath.c_str() );
			}
			break;
		}
	}
}

/**********************************************
* Description: Lays out the dialog
* Author: Rick Caudill
* Date: Thu Mar 18 22:57:43 2004
**********************************************/
void ConvertDialog::Layout()
{
	pcLayoutView = new os::LayoutView( GetBounds(), "L" );
	pcMain = new os::VLayoutNode( "VL" );
	pcMain->SetHAlignment( os::ALIGN_CENTER );
	pcMain->SetVAlignment( os::ALIGN_CENTER );
	pcMain->SetBorders( os::Rect( 5, 5, 5, 5 ) );

	os::VLayoutNode *pcNoteNode = new os::VLayoutNode( "HLNoteNode" );
	pcNoteNode->SetVAlignment( os::ALIGN_CENTER );
	pcNoteNode->AddChild( new os::HLayoutSpacer( "", 25, 25 ) );
	pcNoteNode->AddChild( pcFileString = new os::StringView( os::Rect(), "SVFileString", ( os::String )"File will save as\n'" + GetRelativePath( cFileName ) + ( os::String )".html'." ), os::ALIGN_CENTER );

	os::HLayoutNode *pcFileNode = new os::HLayoutNode( "HLFileNode" );
	pcFileNode->AddChild( pcSaveToString = new os::StringView( os::Rect( 0, 0, 0, 0 ), "SVSaveToString", "Save To:" ) );
	pcFileNode->AddChild( new os::HLayoutSpacer( "", 5, 5 ) );
	pcFileNode->AddChild( pcTVSaveTo = new os::TextView( os::Rect( 0, 0, 0, 0 ), "", "" ), os::CF_FOLLOW_LEFT | os::CF_FOLLOW_TOP | os::CF_FOLLOW_RIGHT );
	pcTVSaveTo->SetMultiLine( false );
	pcFileNode->AddChild( new os::HLayoutSpacer( "", 3, 3 ) );
	pcFileNode->AddChild( pcIBTSaveTo = new os::ImageButton( os::Rect(), "", "", new os::Message( M_REQ_OPEN ), LoadImageFromResource( "folder.png" ), os::ImageButton::IB_TEXT_BOTTOM, true, false, true ), os::CF_FOLLOW_NONE );


	os::HLayoutNode *pcCheckNode = new os::HLayoutNode( "HLCheckNode" );
	pcCheckNode->AddChild( new os::HLayoutSpacer( "", 15, 15 ) );
	pcCheckNode->AddChild( pcCssCheck = new os::CheckBox( os::Rect( 0, 0, 0, 0 ), "", "_Export Css", NULL ) );
	pcCssCheck->SetEnable( false );

	os::HLayoutNode *pcButtonNode = new os::HLayoutNode( "HLButtonNode" );
	pcButtonNode->AddChild( pcConvertButton = new os::Button( os::Rect( 0, 0, 0, 0 ), "convert_but", "Con_vert", new os::Message( M_CONVERT ) ) );
	pcButtonNode->AddChild( new os::HLayoutSpacer( "", 20, 20 ) );
	pcButtonNode->AddChild( pcCancelButton = new os::Button( os::Rect( 0, 0, 0, 0 ), "cancel_but", "_Cancel", new os::Message( M_CONVERT_CANCEL ) ) );
	pcButtonNode->SameWidth( "convert_but", "cancel_but", NULL );

	pcMain->AddChild( new os::VLayoutSpacer( "", 5, 5 ) );
	pcMain->AddChild( pcNoteNode );
	pcMain->AddChild( new os::VLayoutSpacer( "", 5, 5 ) );
	pcMain->AddChild( pcFileNode );
	pcMain->AddChild( new os::VLayoutSpacer( "", 5, 5 ) );
	pcMain->AddChild( pcCheckNode );
	pcMain->AddChild( new os::VLayoutSpacer( "", 5, 5 ) );
	pcMain->AddChild( pcButtonNode );

	pcMain->SameWidth( "HLNoteNode", "HLButtonNode", NULL );

	pcLayoutView->SetRoot( pcMain );
	AddChild( pcLayoutView );
}

/**********************************************
* Description: Sets the tab position
* Author: Rick Caudill
* Date: Thu Mar 18 22:57:43 2004
**********************************************/
void ConvertDialog::SetTabs()
{
	SetDefaultButton( pcConvertButton );
	pcConvertButton->SetShortcut( os::ShortcutKey( "CTRL+V" ) );
	pcCancelButton->SetShortcut( os::ShortcutKey( "CTRL+C" ) );
	pcCssCheck->SetShortcut( os::ShortcutKey( "CTRL+E" ) );
	pcTVSaveTo->SetTabOrder( os::NEXT_TAB_ORDER );
	pcIBTSaveTo->SetTabOrder( os::NEXT_TAB_ORDER );
	pcCssCheck->SetTabOrder( os::NEXT_TAB_ORDER );
	pcConvertButton->SetTabOrder( os::NEXT_TAB_ORDER );
	pcCancelButton->SetTabOrder( os::NEXT_TAB_ORDER );
}

/**********************************************
* Description: Converts the file into a html file
* Author: Rick Caudill
* Date: Thu Mar 18 22:57:43 2004
**********************************************/
void ConvertDialog::Convert()
{
	os::String cPath = "";
	os::String cOutPath = pcTVSaveTo->GetBuffer()[0] == "" ? getenv( "$HOME" ) : pcTVSaveTo->GetBuffer()[0];
	os::Directory *pcDir = new os::Directory( "^/bin/" );
	pcDir->GetPath( &cPath );
	cPath += "/source-highlight";
	delete pcDir;

	if( cOutPath[cOutPath.size() - 1] != '/' )
		cOutPath += "/";
	cOutPath += cFileName;
	cOutPath += ".html";
	os::String cFileType;

	if( cMainFormat == ".cpp" || cMainFormat == ".c" || cMainFormat == ".h" || cMainFormat == ".C" )
	{
		cFileType = "cpp";
	}
	else if( cMainFormat == ".perl" || cMainFormat == ".pm" )
	{
		cFileType = "perl";
	}
	else if( cMainFormat == ".java" )
	{
		cFileType = "java";
	}
	else if( cMainFormat == "Changelog" || cMainFormat == "changelog" )
	{
		cFileType = "changelog";
	}

	if( fork() == 0 )
	{
		set_thread_priority( -1, 0 );
		execlp( cPath.c_str(), cPath.c_str(), "-s", cFileType.c_str(), "-f", "html", "-i", cFileName.c_str(), "-o", cOutPath.c_str(), NULL );
	}
	OkToQuit();
}

/**********************************************
* Description: Called when the window quits
* Author: Rick Caudill
* Date: Thu Mar 18 22:57:43 2004
**********************************************/
bool ConvertDialog::OkToQuit()
{
	Close();
	return false;
}
