//  Sourcery -:-  (C)opyright 2003-2004 Rick Caudill
//
// This is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <util/application.h>
#include <gui/window.h>
#include <util/message.h>
#include <util/resources.h>
#include <gui/image.h>
#include <util/string.h>
#include <gui/menu.h>
#include <storage/file.h>
#include <gui/requesters.h>
#include <gui/imagebutton.h>
#include <gui/popupmenu.h>
#include <gui/filerequester.h>
#include <util/settings.h>
#include <gui/checkmenu.h>
#include <gui/toolbar.h>
#include <storage/nodemonitor.h>

#include <codeview/codeview.h>

#include "editor.h"
#include "statusbar.h"
#include "convertdialog.h"
#include "settings.h"
#include "commonfuncs.h"
#include "gotodialog.h"
#include "finddialog.h"
#include "replacedialog.h"
#include "fileproperties.h"
#include "aboutbox.h"
#include "userinserts.h"
#include "browser.h"
#include "settingsdialog.h"
#include "toolsdialog.h"

class SourceryFile;
struct ToolHandle;

class MainWindow : public os::Window
{
public:
	MainWindow( AppSettings * pcSettings, const os::String &cString );
	virtual ~MainWindow();
	virtual void HandleMessage( os::Message * pcMessage );
	virtual void TimerTick( int nID );
	virtual void DispatchMessage( os::Message * pcMessage, os::Handler * pcHandler );
	virtual bool OkToQuit();

	bool Find( const os::String &pcString, bool bDown, bool bCaseSensitive, bool bExtended );
	void GoTo( uint nWidth, uint nHeight );
	void ZoomOut();
	void ZoomIn();

private:
	void SetupMenus();
	void SetupToolBar();
	void SetupStatusBar();
	void Load( const os::String &cString, bool bOpenedByMain = false );
	void Save( const os::String &cFile );
	void CloseFile();
	void UpdateTitle();
	void UpdateStatus( const os::String &cString, bigtime_t nTime );
	void LoadSettings();
	void LoadDefaults();
	void SaveSettings();
	void LoadFormat();
	void LaunchConvertor();
	void OpenCorrespondingFile();
	void ShowError( int nCode );
	void AddLanguageInformation();
	void ReloadCustomItems( bool bFirstTime );
	void ReloadCustomTools( bool bFirstTime );
	void LoadNewFormat( const os::String &cItemName );
	void ReloadRecentMenu();
	void SetRecentList( const os::String &cPath );
	void Init();

	os::Message *LoadCustomItemMessage( const os::String &cInsertedText );
	os::Message *LoadCustomToolsMessage( ToolHandle * pcHandle );
	os::Menu *GetInsertMenu();
	os::Menu *GetFindMenu();
	os::ToolBar *m_pcToolBar;
	os::Menu *m_pcPopMenu;
	os::Menu *m_pcContextMenu;
	os::Menu *m_pcMenu;
	os::Menu *m_pcAppMenu;
	os::Menu *m_pcFileMenu;
	os::Menu *m_pcEditMenu;
	os::Menu *m_pcViewMenu;
	os::Menu *m_pcFindMenu;
	os::Menu *m_pcHelpMenu;
	os::Menu *m_pcInsertMenu;
	os::Menu *m_pcInsertMenuTwo;
	os::Menu *m_pcCommentMenu;
	os::Menu *m_pcLanguageMenu;
	os::Menu *m_pcInsertCustomMenu;
	os::Menu *m_pcDateMenu;
	os::Menu *m_pcToolMenu;
	os::Menu *m_pcTemplateMenu;
	os::Menu *pcRecentMenu;
	os::Menu *m_pcFormatMenu;
	os::CheckMenu *m_pcMainToolBarCheck;
	os::CheckMenu *m_pcFindToolBarCheck;
	os::CheckMenu *m_pcShowLineNumberCheck;
	os::FileRequester *pcOpenRequester;
	os::FileRequester *pcSaveRequester;
	os::Settings *pcSettings;
	os::String cFile;
	os::String cConvertor;
	os::String cSearchText;
	os::NodeMonitor *pcMonitorFile;
	os::Message *pcPassedMessage;
	os::Event * pcGetFileEvent;
	os::Event * pcMakeFocusEvent;
	std::vector < os::String >GetRecentList();

	Editor *pcEditor;
	StatusBar *m_pcStatusBar;
	APIBrowserWindow *m_pcBrowserWindow;
	SettingsDialog *m_pcSettingsDialog;
	ConvertDialog *pcConvertDialog;
	GoToDialog *pcGoToDialog;
	FindDialog *pcFindDialog;
	FileProp *pcFileProp;
	ReplaceDialog *pcReplaceDialog;
	AboutBoxWindow *pcAbout;
	UserInserts *pcUserInserts;
	ToolsWindow *pcToolsWindow;
	AppSettings *pcMainSettings;
	SourceryFile *sFile;

	bool bChanged;
	bool bNamed;
	bool bFound;
	bool bSearchDown;
	bool bSearchCase;
	bool bSearchExtended;
	int nMonitor;
};

#endif
