//  Sourcery -:-  (C)opyright 2003-2004 Rick Caudill
//
// This is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#ifndef _SETTINGS_DIALOG_H
#define _SETTINGS_DIALOG_H

#include <cmath>

#include <gui/window.h>
#include <gui/stringview.h>
#include <gui/view.h>
#include <gui/font.h>
#include <gui/frameview.h>
#include <gui/button.h>
#include <gui/checkbox.h>
#include <gui/splitter.h>
#include <gui/textview.h>
#include <gui/listview.h>
#include <gui/font.h>
#include <gui/iconview.h>
#include <gui/listview.h>
#include <gui/spinner.h>
#include <gui/colorselector.h>

#include "settings.h"
#include "colorbutton.h"

class DialogHighlighter : public os::View
{
public:
	DialogHighlighter( const os::String &cName );
	virtual void Paint( const os::Rect & cUpdateRect );

private:
	os::String cHighlight;
};

class ColorSelectorWindow : public os::Window
{
public:
	ColorSelectorWindow( int nType, os::Color32_s sColor, Window * pcWindow );
	virtual void HandleMessage( os::Message * pcMessage );
	virtual bool OkToQuit();

	void Layout();

private:
	os::ColorSelector * pcSelector;
	os::Button *pcOkButton;
	os::Button *pcCancelButton;
	os::LayoutView *pcLayoutView;
	os::Window *pcParentWindow;
	os::Color32_s sControlColor;

	int nControl;
};

class SettingsDialogGeneral : public os::LayoutView
{
public:
	SettingsDialogGeneral();
	virtual ~SettingsDialogGeneral();

	void LayoutSystem();
	void Init( AppSettings * pcSettings );
	void SaveSettings( AppSettings * pcSettings );

private:
	os::VLayoutNode * pcRoot;
	os::CheckBox *pcSplashCheck;
	os::CheckBox *pcAdvancedGoToCheck;
	os::CheckBox *pcBackupCheck;
	os::CheckBox *pcMonitorCheck;
	os::CheckBox *pcSaveCursorCheck;
	os::CheckBox *pcFoldCheck;
	os::CheckBox *pcSaveWindowCheck;
	os::CheckBox *pcConvertLFCheck;
	os::CheckBox *pcFileMinCheck;
	os::CheckBox *pcLineCheck;

	DialogHighlighter *pcHighlighter;
};

class SettingsDialogFont : public os::LayoutView
{
public:
	SettingsDialogFont( struct FontHandle * pcHandle );
	virtual ~SettingsDialogFont();

	void LoadFontTypes();
	void FontHasChanged();
	void FaceHasChanged();
	void SizeHasChanged();
	void AliasChanged();
	void ShearChanged();
	void RotateChanged();

	FontHandle *GetFontHandle();

private:
	void SetupFonts();
	void LayoutSystem();
	void InitialFont();

	os::VLayoutNode *pcRoot;
	os::ListView *pcFontListView;
	os::ListView *pcFontFaceListView;
	os::ListView *pcFontSizeListView;
	os::CheckBox *pcAntiCheck;
	os::StringView *pcTextStringView;
	os::FrameView *pcTextFrameView;
	os::StringView *pcRotationString;
	os::StringView *pcShearString;
	os::Spinner *pcRotationSpinner;
	os::Spinner *pcShearSpinner;

	struct FontHandle *pcFontHandle;
};

class SettingsDialogShortcuts : public os::LayoutView
{
public:
	SettingsDialogShortcuts();
	virtual ~SettingsDialogShortcuts();

private:
	os::VLayoutNode * pcRoot;
	os::StringView *pcError;
};

class SettingsDialogPlugins : public os::LayoutView
{
public:
	SettingsDialogPlugins();
	virtual ~SettingsDialogPlugins();

private:
	os::VLayoutNode * pcRoot;
	os::StringView *pcError;
};

class SettingsDialogColors : public os::LayoutView
{
public:
	SettingsDialogColors();
	virtual ~SettingsDialogColors();

	void SetColors( AppSettings * pcSettings );
	void LaunchColorSelectorWindow( int nColor, os::Color32_s sColor, os::Window * pcWindow );
	void CancelColorSelectorWindow();
	void ColorSelectorWindowRequested( os::Color32_s sColor, int nControl );
	void ColorChanged( int nColor, os::Color32_s sColor );

private:
	void LayoutSystem();
	void Init();

	os::VLayoutNode *pcRoot;
	os::StringView *pcBackColorString;
	os::StringView *pcNumberColorString;
	os::StringView *pcHighlightColorString;
	os::StringView *pcTextColorString;
	os::Color32_s sBackColor;
	os::Color32_s sNumberColor;
	os::Color32_s sHighlightColor;
	os::Color32_s sTextColor;

	ColorButton *pcBackColorButton;
	ColorButton *pcNumberColorButton;
	ColorButton *pcHighlightColorButton;
	ColorButton *pcTextColorButton;
	ColorSelectorWindow *pcColorSelectorWindow;
};

class SettingsDialog : public os::Window
{
public:
	SettingsDialog( os::Window * pcParent, AppSettings * pcAppSettings );
	virtual ~SettingsDialog();
	virtual void HandleMessage( os::Message * pcMessage );
	virtual bool OkToQuit( void );

private:
	void AddIcons();
	void Layout();
	void Save();

	os::FrameView *pcView;
	os::Button *pcSaveButton, *pcCancelButton, *pcDefaultButton;
	//os::LayoutView* pcSettingsView;
	os::LayoutView *pcButtonView;
	os::HLayoutNode *pcButtonNode;
	os::HLayoutNode *pcLayoutNode;
	os::HLayoutNode *pcRoot;
	os::IconView *pcIconView;
	os::Window *pcParentWindow;
	os::VLayoutNode *pcClearNode;
	os::Splitter *pcSplitter;

	SettingsDialogGeneral *pcGeneral;
	SettingsDialogFont *pcFonts;
	SettingsDialogPlugins *pcPlugins;
	SettingsDialogShortcuts *pcShortcuts;
	SettingsDialogColors *pcColors;
	AppSettings *pcSettings;
};
#endif //_SETTINGS_DIALOG_H
