//  Sourcery -:-  (C)opyright 2003-2004 Rick Caudill 
//
// This is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//  
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "toolsdialog.h"
#include "messages.h"
#include "toolhandle.h"

/*************************************************************
* Description: Simple window that lets you edit/create a new tool
* Author: Rick Caudill 
* Date: Sun Oct 24 2004 03:57:23 
*************************************************************/
ToolsEditWindow::ToolsEditWindow( os::Window * pcParent, os::Message * pcMessage ) : os::Window( os::Rect( 0, 0, 1, 1 ), "tools_edit", "New Command...", os::WND_NOT_RESIZABLE )
{
	if( pcMessage != NULL )
	{
		pcMessage->FindString( "command", &cCommand );
		pcMessage->FindString( "name", &cName );
		pcMessage->FindString( "arguments", &cArguments );
		pcMessage->FindInt( "selected", &nSelected );
	}
	else
	{
		cCommand = "";
		cName = "";
		cArguments = "";
		nSelected = -1;
	}
	pcParentWindow = pcParent;
	Layout();
	SetTabs();
	ResizeTo( pcLayoutView->GetPreferredSize( false ) );
}

/*************************************************************
* Description: Sets the taborder of all elements
* Author: Rick Caudill 
* Date: Sun Oct 24 2004 04:00:25 
*************************************************************/
void ToolsEditWindow::SetTabs()
{
	pcNameText->SetTabOrder( os::NEXT_TAB_ORDER );
	pcCommandText->SetTabOrder( os::NEXT_TAB_ORDER );
	pcArgumentsText->SetTabOrder( os::NEXT_TAB_ORDER );
	pcOkButton->SetTabOrder( os::NEXT_TAB_ORDER );
	pcCancelButton->SetTabOrder( os::NEXT_TAB_ORDER );
	SetDefaultButton( pcOkButton );
	SetFocusChild( pcNameText );
}

/*************************************************************
* Description: Changes to the current tool
* Author: Rick Caudill 
* Date: Sun Oct 24 2004 05:22:05 
*************************************************************/
void ToolsEditWindow::ChangeTo( os::Message * pcMessage )
{
	if( pcMessage != NULL )
	{
		pcMessage->FindString( "command", &cCommand );
		pcMessage->FindString( "name", &cName );
		pcMessage->FindString( "arguments", &cArguments );
		pcMessage->FindInt( "selected", &nSelected );
	}
	pcNameText->Set( cName.c_str() );
	pcCommandText->Set( cCommand.c_str() );
	pcArgumentsText->Set( cArguments.c_str() );
}

/*************************************************************
* Description: Lays the editor window out
* Author: Rick Caudill 
* Date: Sun Oct 24 2004 04:00:50 
*************************************************************/
void ToolsEditWindow::Layout()
{
	pcLayoutView = new os::LayoutView( GetBounds(), "main_layout_view", NULL, os::CF_FOLLOW_ALL );
	os::VLayoutNode *pcRoot = new os::VLayoutNode( "main_layout" );
	pcRoot->SetBorders( os::Rect( 5, 5, 5, 10 ) );

	os::HLayoutNode *pcNameNode = new os::HLayoutNode( "name_node" );
	pcNameNode->AddChild( pcNameString = new os::StringView( os::Rect(), "name_string", "Title:" ) );
	pcNameNode->AddChild( new os::HLayoutSpacer( "", 5, 2 ) );
	pcNameNode->AddChild( pcNameText = new os::TextView( os::Rect(), "name_text", cName.c_str() ) );

	os::HLayoutNode *pcCommandNode = new os::HLayoutNode( "command_node" );
	pcCommandNode->AddChild( pcCommandString = new os::StringView( os::Rect(), "command_string", "Command:" ) );
	pcCommandNode->AddChild( new os::HLayoutSpacer( "", 5, 2 ) );
	pcCommandNode->AddChild( pcCommandText = new os::TextView( os::Rect(), "command_text", cCommand.c_str() ) );

	os::HLayoutNode *pcArgumentsNode = new os::HLayoutNode( "arguments_node" );
	pcArgumentsNode->AddChild( pcArgumentsString = new os::StringView( os::Rect(), "arguments_string", "Arguments:" ) );
	pcArgumentsNode->AddChild( new os::HLayoutSpacer( "", 5, 2 ) );
	pcArgumentsNode->AddChild( pcArgumentsText = new os::TextView( os::Rect(), "arguments_text", cArguments.c_str() ) );

	os::HLayoutNode *pcOtherButtonNode = new os::HLayoutNode( "other_buttons" );
	pcOtherButtonNode->SetHAlignment( os::ALIGN_RIGHT );
	pcOtherButtonNode->SetVAlignment( os::ALIGN_RIGHT );
	pcOtherButtonNode->AddChild( new os::HLayoutSpacer( "", 100 ) );
	pcOtherButtonNode->AddChild( pcOkButton = new os::Button( os::Rect(), "save_but", "_Save", new os::Message( M_TOOLS_SAVE ) ) );
	pcOtherButtonNode->AddChild( new os::HLayoutSpacer( "", 5, 5 ) );
	pcOtherButtonNode->AddChild( pcCancelButton = new os::Button( os::Rect(), "cancel_but", "_Cancel", new os::Message( M_TOOLS_CANCEL ) ) );
	pcOtherButtonNode->SameWidth( "save_but", "cancel_but", NULL );

	pcRoot->AddChild( pcNameNode );
	pcRoot->AddChild( new os::VLayoutSpacer( "", 5, 5 ) );
	pcRoot->AddChild( pcCommandNode );
	pcRoot->AddChild( new os::VLayoutSpacer( "", 5, 5 ) );
	pcRoot->AddChild( pcArgumentsNode );
	pcRoot->AddChild( new os::VLayoutSpacer( "", 5, 5 ) );
	pcRoot->AddChild( pcOtherButtonNode );
	pcRoot->SameWidth( "arguments_string", "command_string", "name_string", NULL );
	pcLayoutView->SetRoot( pcRoot );
	AddChild( pcLayoutView );
}

/*************************************************************
* Description: Handles all messages passed through the edit window
* Author: Rick Caudill 
* Date: Sun Oct 24 2004 04:01:04 
*************************************************************/
void ToolsEditWindow::HandleMessage( os::Message * pcMessage )
{
	switch ( pcMessage->GetCode() )
	{
		case M_TOOLS_SAVE:
		{
			pcSaveMessage = new os::Message( M_TOOLS_QUIT );
			pcSaveMessage->AddBool( "saved", true );
			pcSaveMessage->AddString( "command", pcCommandText->GetBuffer()[0] );
			pcSaveMessage->AddString( "name", pcNameText->GetBuffer()[0] );
			pcSaveMessage->AddString( "arguments", pcArgumentsText->GetBuffer()[0] );
			pcSaveMessage->AddInt32( "selected", nSelected );
			OkToQuit();
			break;
		}
		case M_TOOLS_CANCEL:
		{
			pcSaveMessage = new os::Message( M_TOOLS_QUIT );
			OkToQuit();
			break;
		}
	}
}

/*************************************************************
* Description: Called when the window is closed with the close/cancel buttons
* Author: Rick Caudill 
* Date: Sun Oct 24 2004 04:01:38 
*************************************************************/
bool ToolsEditWindow::OkToQuit()
{
	pcParentWindow->PostMessage( pcSaveMessage, pcParentWindow );
	return true;
}


/*************************************************************
* Description: ToolsWindow lets the user have external tools they can
*              from within Sourcery.
* Author: Rick Caudill 
* Date: Sun Oct 24 2004 04:03:07 
*************************************************************/
ToolsWindow::ToolsWindow( os::Window * pcParent ) : os::Window( os::Rect( 0, 0, 1, 1 ), "external_tools", "External Tools..", os::WND_NOT_RESIZABLE )
{
	pcParentWindow = pcParent;
	pcToolsEditWindow = NULL;
	bSaved = false;
	Layout();
	SetTabs();
	ResizeTo( os::Point( pcLayoutView->GetPreferredSize( false ).x + 90, pcLayoutView->GetPreferredSize( false ).y + 100 ) );
}

/*************************************************************
* Description: Lays the window out
* Author: Rick Caudill 
* Date: Sun Oct 24 2004 04:04:07 
*************************************************************/
void ToolsWindow::Layout()
{
	pcLayoutView = new os::LayoutView( GetBounds(), "main_layoutview", NULL, os::CF_FOLLOW_ALL );
	os::VLayoutNode *pcRoot = new os::VLayoutNode( "main_layout" );
	pcRoot->SetBorders( os::Rect( 5, 5, 5, 10 ) );

	os::HLayoutNode *pcListNode = new os::HLayoutNode( "list_node" );
	pcListNode->AddChild( pcListView = new os::ListView( os::Rect( 0, 0, 0, 0 ), "edit_list", os::CF_FOLLOW_ALL ) );
	pcListView->InsertColumn( "Name", 100 );
	pcListView->InsertColumn( "Command", 100 );
	pcListView->InsertColumn( "Arguments", 100 );
	pcListNode->SetWeight( 120 );

	os::HLayoutNode *pcListButtonNode = new os::HLayoutNode( "list_buttons" );
	pcListButtonNode->AddChild( new os::HLayoutSpacer( "", 20 ) );
	pcListButtonNode->AddChild( pcNewButton = new os::Button( os::Rect(), "new_but", "_New", new os::Message( M_TOOLS_NEW ), os::CF_FOLLOW_LEFT | os::CF_FOLLOW_RIGHT ) );
	pcListButtonNode->AddChild( new os::HLayoutSpacer( "", 10 ) );
	pcListButtonNode->AddChild( pcEditButton = new os::Button( os::Rect(), "edit_but", "_Edit", new os::Message( M_TOOLS_EDIT ), os::CF_FOLLOW_LEFT | os::CF_FOLLOW_RIGHT ) );
	pcListButtonNode->AddChild( new os::HLayoutSpacer( "", 10 ) );
	pcListButtonNode->AddChild( pcDeleteButton = new os::Button( os::Rect(), "del_but", "_Delete", new os::Message( M_TOOLS_DELETE ), os::CF_FOLLOW_LEFT | os::CF_FOLLOW_RIGHT ) );
	pcListButtonNode->AddChild( new os::HLayoutSpacer( "", 20 ) );
	pcListButtonNode->SameWidth( "del_but", "edit_but", "new_but", NULL );

	os::HLayoutNode *pcOtherButtonNode = new os::HLayoutNode( "other_buttons" );
	pcOtherButtonNode->SetHAlignment( os::ALIGN_RIGHT );
	pcOtherButtonNode->SetVAlignment( os::ALIGN_RIGHT );
	pcOtherButtonNode->AddChild( new os::HLayoutSpacer( "", 100 ) );
	pcOtherButtonNode->AddChild( pcSaveButton = new os::Button( os::Rect(), "save_but", "_Save", new os::Message( M_TOOLS_SAVE ) ) );
	pcOtherButtonNode->AddChild( new os::HLayoutSpacer( "", 5, 5 ) );
	pcOtherButtonNode->AddChild( pcCancelButton = new os::Button( os::Rect(), "cancel_but", "_Cancel", new os::Message( M_TOOLS_CANCEL ) ) );
	pcOtherButtonNode->SameWidth( "save_but", "cancel_but", NULL );

	pcRoot->AddChild( pcListNode );
	pcRoot->AddChild( pcListButtonNode );
	pcRoot->AddChild( new os::VLayoutSpacer( "", 10 ) );
	pcRoot->AddChild( pcOtherButtonNode );
	pcLayoutView->SetRoot( pcRoot );
	AddChild( pcLayoutView );
	LoadTools();
}

/*************************************************************
* Description: Sets the tab order of everything
* Author: Rick Caudill 
* Date: Sun Oct 24 2004 04:04:23 
*************************************************************/
void ToolsWindow::SetTabs()
{
	pcNewButton->SetTabOrder( os::NEXT_TAB_ORDER );
	pcEditButton->SetTabOrder( os::NEXT_TAB_ORDER );
	pcDeleteButton->SetTabOrder( os::NEXT_TAB_ORDER );
	pcSaveButton->SetTabOrder( os::NEXT_TAB_ORDER );
	pcCancelButton->SetTabOrder( os::NEXT_TAB_ORDER );
	SetDefaultButton( pcSaveButton );
	SetFocusChild( pcNewButton );
}

/*************************************************************
* Description: handles all messages passed through the window
* Author: Rick Caudill 
* Date: Sun Oct 24 2004 04:04:37 
*************************************************************/
void ToolsWindow::HandleMessage( os::Message * pcMessage )
{
	switch ( pcMessage->GetCode() )
	{
		case M_TOOLS_SAVE:
		{
			bSaved = true;
			SaveTools();
			OkToQuit();
			break;
		}
		case M_TOOLS_CANCEL:
		{
			OkToQuit();
			break;
		}
		case M_TOOLS_EDIT:
		{
			if( pcListView->GetLastSelected() > -1 )
			{
				os::ListViewStringRow *pcRow = ( os::ListViewStringRow * ) pcListView->GetRow( pcListView->GetLastSelected() );
				os::Message *pcMsg = new os::Message();
				pcMsg->AddString( "name", pcRow->GetString( 0 ) );
				pcMsg->AddString( "command", pcRow->GetString( 1 ) );
				pcMsg->AddString( "arguments", pcRow->GetString( 2 ) );
				pcMsg->AddInt32( "selected", pcListView->GetLastSelected() );
				if( pcToolsEditWindow == NULL )
				{
					pcToolsEditWindow = new ToolsEditWindow( this, pcMsg );
					pcToolsEditWindow->CenterInWindow( this );
					pcToolsEditWindow->Show();
					pcToolsEditWindow->MakeFocus();
				}
				else
				{
					pcToolsEditWindow->ChangeTo( pcMsg );
					pcToolsEditWindow->MakeFocus( true );
				}
			}
			break;
		}
		case M_TOOLS_DELETE:
		{
			uint nSelected = pcListView->GetLastSelected();
			if( nSelected >= 0 )
			{
				pcListView->RemoveRow( nSelected, true );
			}
			break;
		}
		case M_TOOLS_NEW:
		{
			if( pcToolsEditWindow == NULL )
			{
				pcToolsEditWindow = new ToolsEditWindow( this, NULL );
				pcToolsEditWindow->CenterInWindow( this );
				pcToolsEditWindow->Show();
				pcToolsEditWindow->MakeFocus();
			}
			else
			{
				pcToolsEditWindow->MakeFocus( true );
			}
			break;
		}
		case M_TOOLS_QUIT:
		{
			if( pcToolsEditWindow != NULL )
			{
				bool bSavedTools;
				if( pcMessage->FindBool( "saved", &bSavedTools ) == 0 )
				{
					os::String cName, cCommand, cArguments;
					int nSelected = 0;
					pcMessage->FindString( "command", &cCommand );
					pcMessage->FindString( "name", &cName );
					pcMessage->FindString( "arguments", &cArguments );
					pcMessage->FindInt( "selected", &nSelected );
					if( nSelected == -1 ) //this means it is a new tool
					{
						os::ListViewStringRow *pcRow = new os::ListViewStringRow();
						pcRow->AppendString( cName );
						pcRow->AppendString( cCommand );
						pcRow->AppendString( cArguments );
						pcListView->InsertRow( pcRow );
					}
					else //this means that a certain tool was edited
					{
						os::ListViewStringRow *pcRow = ( os::ListViewStringRow * ) pcListView->GetRow( nSelected );
						pcRow->SetString( 0, cName );
						pcRow->SetString( 1, cCommand );
						pcRow->SetString( 2, cArguments );
					}
				}
				pcToolsEditWindow->Close();
				pcToolsEditWindow = NULL;
			}
			break;
		}
	}
}

/*************************************************************
* Description: Loads all the tools
* Author: Rick Caudill 
* Date: Sun Oct 24 2004 04:05:33 
*************************************************************/
void ToolsWindow::LoadTools()
{
	try
	{
		os::String zPath = getenv( "HOME" );
		zPath += "/Settings/Sourcery/Tools";
		os::Settings * pcSettings = new os::Settings( new os::File( zPath, O_RDWR | O_CREAT ) );
		pcSettings->Load();

		for( int i = 0; i < pcSettings->CountNames(); i++ )
		{
			ToolHandle *pcHandle;
			size_t nToolHandleSize;
			pcSettings->FindData( pcSettings->GetName( i ).c_str(), os::T_RAW, ( const void ** )&pcHandle, &nToolHandleSize );
			os::ListViewStringRow *pcRow = new os::ListViewStringRow();
			pcRow->AppendString( pcHandle->zName );
			pcRow->AppendString( pcHandle->zCommand );
			pcRow->AppendString( pcHandle->zArguments );
			pcListView->InsertRow( pcRow );
		}
		delete pcSettings;
	}

	catch( ... )
	{
	}
}

/*************************************************************
* Description: Saves all the tools
* Author: Rick Caudill 
* Date: Sun Oct 24 2004 04:06:05 
*************************************************************/
void ToolsWindow::SaveTools()
{
	try
	{
		os::String zPath = getenv( "HOME" );
		zPath += "/Settings/Sourcery/Tools";
		os::Settings * pcSettings = new os::Settings( new os::File( zPath ) );

		for( int i = 0; i < pcSettings->CountNames(); i++ )
		{
			pcSettings->RemoveName( pcSettings->GetName( i ).c_str() );
		}

		for( uint i = 0; i < pcListView->GetRowCount(); i++ )
		{
			ToolHandle *pcHandle = new ToolHandle();
			os::ListViewStringRow *pcRow = ( os::ListViewStringRow * ) pcListView->GetRow( i );
			strcpy( pcHandle->zName, pcRow->GetString( 0 ).c_str() );
			strcpy( pcHandle->zCommand, pcRow->GetString( 1 ).c_str() );
			strcpy( pcHandle->zArguments, pcRow->GetString( 2 ).c_str() );
			pcSettings->SetData( pcRow->GetString( 0 ).c_str(), os::T_RAW, ( void * )pcHandle, sizeof( ToolHandle ) );
		}
		pcSettings->Save();
		delete( pcSettings );
	}
	catch( ... )
	{
	}
}

/*************************************************************
* Description: Called when tools window wants to quit
* Author: Rick Caudill 
* Date: Sun Oct 24 2004 04:05:11 
*************************************************************/
bool ToolsWindow::OkToQuit()
{
	os::Message *pcMessage = new os::Message( M_TOOLS_QUIT );
	pcMessage->AddBool( "saved", &bSaved );
	pcParentWindow->PostMessage( pcMessage, pcParentWindow );
	return false;
}
