// Sourcery -:-  (C)opyright 2003-2004 Rick Caudill
//
// This is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "userinserts.h"
#include "messages.h"
#include "commonfuncs.h"

#include <gui/imagebutton.h>
#include <gui/requesters.h>

UserInsertsEditWindow::UserInsertsEditWindow( os::Window * pcParent, bool bNew ) : os::Window( os::Rect( 0, 0, 200, 200 ), "edit_win", "New Insert...", os::WND_NOT_RESIZABLE )
{
	pcParentWindow = pcParent;
	bIsNewWindow = bNew;
	Layout();
	ResizeTo( pcView->GetPreferredSize( false ) );
}

void UserInsertsEditWindow::Layout()
{
	pcView = new os::LayoutView( GetBounds(), "main_layout", NULL, os::CF_FOLLOW_ALL );
	os::VLayoutNode *pcRoot = new os::VLayoutNode( "main_layout_node" );
	pcRoot->SetBorders( os::Rect( 5, 5, 10, 5 ) );

	os::HLayoutNode *pcNameNode = new os::HLayoutNode( "name_node" );
	pcNameNode->AddChild( pcNameString = new os::StringView( os::Rect(), "name_string", "Name:", os::ALIGN_LEFT, os::CF_FOLLOW_NONE ) );
	pcNameNode->AddChild( new os::HLayoutSpacer( "", 5 ) );
	pcNameNode->AddChild( pcNameText = new os::TextView( os::Rect(), "text_name", "", os::CF_FOLLOW_NONE ), 100 );

	os::HLayoutNode *pcTextNode = new os::HLayoutNode( "text_node" );
	pcTextNode->AddChild( pcInsertedString = new os::StringView( os::Rect(), "insert_string", "Text:", os::ALIGN_LEFT, os::CF_FOLLOW_NONE ) );
	pcTextNode->AddChild( new os::HLayoutSpacer( "", 5 ) );
	pcTextNode->AddChild( pcInsertedText = new os::TextView( os::Rect(), "text_insert", "", os::CF_FOLLOW_ALL ) );
	pcInsertedText->SetMultiLine( true );
	pcInsertedText->SetMaxPreferredSize( 25, 8 );
	pcInsertedText->SetMinPreferredSize( 25, 8 );

	os::HLayoutNode *pcButtonNode = new os::HLayoutNode( "button_node" );
	pcButtonNode->SetHAlignment( os::ALIGN_RIGHT );
	pcButtonNode->AddChild( new os::HLayoutSpacer( "", 170 ) );
	pcButtonNode->AddChild( pcOkButton = new os::Button( os::Rect(), "ok_but", "_OK", new os::Message( M_USER_INSERT_SAVE ), os::CF_FOLLOW_NONE ) );
	pcButtonNode->AddChild( new os::HLayoutSpacer( "", 5 ) );
	pcButtonNode->AddChild( pcCancelButton = new os::Button( os::Rect(), "cancel_but", "_Cancel", new os::Message( M_USER_INSERT_CANCEL ), os::CF_FOLLOW_NONE ) );
	pcButtonNode->SameWidth( "ok_but", "cancel_but", NULL );

	pcRoot->AddChild( pcNameNode );
	pcRoot->AddChild( new os::VLayoutSpacer( "", 5 ) );
	pcRoot->AddChild( pcTextNode );
	pcRoot->AddChild( new os::VLayoutSpacer( "", 5 ) );
	pcRoot->AddChild( pcButtonNode );
	pcRoot->AddChild( new os::VLayoutSpacer( "", 5 ) );
	pcRoot->SameWidth( "insert_string", "name_string", NULL );
	pcRoot->SameWidth( "text_name", "text_insert", NULL );
	pcView->SetRoot( pcRoot );
	AddChild( pcView );

	pcNameText->SetTabOrder( os::NEXT_TAB_ORDER );
	pcInsertedText->SetTabOrder( os::NEXT_TAB_ORDER );
	pcOkButton->SetTabOrder( os::NEXT_TAB_ORDER );
	pcCancelButton->SetTabOrder( os::NEXT_TAB_ORDER );
	SetFocusChild( pcNameText );
	SetDefaultButton( pcOkButton );
}

void UserInsertsEditWindow::HandleMessage( os::Message * pcMessage )
{
	switch ( pcMessage->GetCode() )
	{
		case M_USER_INSERT_SAVE:
		{
			os::Message *pcMsg;
			os::String cInsertedText = "";
			for( uint i = 0; i < pcInsertedText->GetBuffer().size(); i++ )
			{
				cInsertedText += pcInsertedText->GetBuffer()[i];
				cInsertedText += "\n";
			}

			if( bIsNewWindow )
			{
				pcMsg = new os::Message( M_USER_INSERT_NEW_PASSED );
				pcMsg->AddString( "name", pcNameText->GetBuffer()[0] );
				pcMsg->AddString( "text", cInsertedText );
			}
			else
			{
				pcMsg = new os::Message( M_USER_INSERT_EDIT_PASSED );
				pcMsg->AddString( "name", pcNameText->GetBuffer()[0] );
				pcMsg->AddString( "text", cInsertedText );
			}
			pcParentWindow->PostMessage( pcMsg, pcParentWindow );
			OkToQuit();
			break;
		}
		case M_USER_INSERT_CANCEL:
		{
			OkToQuit();
			break;
		}
	}
}

void UserInsertsEditWindow::Insert( const os::String &cName, const os::String &cText )
{
	pcNameText->Set( cName.c_str() );
	pcInsertedText->Set( cText.c_str() );
	SetTitle( os::String ( "Editing: " ) + cName );
	bIsNewWindow = false;
}

bool UserInsertsEditWindow::OkToQuit()
{
	Close();
	return false;
}


UserInserts::UserInserts( os::Window * pcParent ) : os::Window( os::Rect( 0, 0, 200, 200 ), "User Inserts...", "User Inserts...", os::WND_NOT_RESIZABLE )
{
	pcParentWindow = pcParent;
	Layout();
	LoadInserts();
	ResizeTo( os::Point( pcView->GetPreferredSize( false ).x, pcView->GetPreferredSize( false ).y + 100 ) );
}

void UserInserts::Layout()
{
	pcView = new os::LayoutView( GetBounds(), "main_layout_view", NULL );
	os::VLayoutNode *pcRoot = new os::VLayoutNode( "main_layout" );
	pcRoot->SetBorders( os::Rect( 5, 5, 5, 10 ) );

	os::HLayoutNode *pcFormatsNode = new os::HLayoutNode( "formats_node" );
	os::BitmapImage *pcArrows = LoadImageFromResource( "arrows.png" );
	pcFormatsNode->AddChild( pcFormatsString = new os::StringView( os::Rect(), "", "%D - Displays the date\n%T - Displays the time", os::ALIGN_RIGHT, os::CF_FOLLOW_LEFT | os::CF_FOLLOW_TOP, os::WID_WILL_DRAW | os::WID_TRANSPARENT ) );
	pcFormatsNode->AddChild( new os::HLayoutSpacer( "", 2 ) );
	pcFormatsNode->AddChild( new os::ImageButton( os::Rect(), "", "", new os::Message( M_USER_INSERT_MACROS ), pcArrows, os::ImageButton::IB_TEXT_BOTTOM, true, false, true ) );
	pcFormatsNode->AddChild( new os::HLayoutSpacer( "", 7 ) );

	os::HLayoutNode *pcListNode = new os::HLayoutNode( "list_node" );
	pcListNode->AddChild( pcInsertsList = new os::ListView( os::Rect( 0, 0, 0, 0 ), "edit_list", os::CF_FOLLOW_ALL ) );
	pcInsertsList->InsertColumn( "Name", (int)GetBounds().Width() / 2 - 10 );
	pcInsertsList->InsertColumn( "Text Inserted", (int)GetBounds().Width() / 2 - 10 );
	pcListNode->SetWeight( 120 );

	os::HLayoutNode *pcListButtonNode = new os::HLayoutNode( "list_buttons" );
	pcListButtonNode->AddChild( new os::HLayoutSpacer( "", 20 ) );
	pcListButtonNode->AddChild( pcNewButton = new os::Button( os::Rect(), "new_but", "_New", new os::Message( M_USER_INSERT_NEW ), os::CF_FOLLOW_LEFT | os::CF_FOLLOW_RIGHT ) );
	pcListButtonNode->AddChild( new os::HLayoutSpacer( "", 10 ) );
	pcListButtonNode->AddChild( pcEditButton = new os::Button( os::Rect(), "edit_but", "_Edit", new os::Message( M_USER_INSERT_EDIT ), os::CF_FOLLOW_LEFT | os::CF_FOLLOW_RIGHT ) );
	pcListButtonNode->AddChild( new os::HLayoutSpacer( "", 10 ) );
	pcListButtonNode->AddChild( pcDeleteButton = new os::Button( os::Rect(), "del_but", "_Delete", new os::Message( M_USER_INSERT_DEL ), os::CF_FOLLOW_LEFT | os::CF_FOLLOW_RIGHT ) );
	pcListButtonNode->AddChild( new os::HLayoutSpacer( "", 20 ) );
	pcListButtonNode->SameWidth( "del_but", "edit_but", "new_but", NULL );

	os::HLayoutNode *pcOtherButtonNode = new os::HLayoutNode( "other_buttons" );
	pcOtherButtonNode->AddChild( new os::HLayoutSpacer( "", 100 ) );
	pcOtherButtonNode->AddChild( pcOkButton = new os::Button( os::Rect(), "save_but", "_Save", new os::Message( M_USER_INSERT_SAVE ) ) );
	pcOtherButtonNode->AddChild( new os::HLayoutSpacer( "", 5 ) );
	pcOtherButtonNode->AddChild( pcCancelButton = new os::Button( os::Rect(), "cancel_but", "_Cancel", new os::Message( M_USER_INSERT_CANCEL ) ) );
	pcOtherButtonNode->SameWidth( "save_but", "cancel_but", NULL );

	pcRoot->AddChild( pcFormatsNode );
	pcRoot->AddChild( pcListNode );
	pcRoot->AddChild( pcListButtonNode );
	pcRoot->AddChild( new os::VLayoutSpacer( "", 10 ) );
	pcRoot->AddChild( pcOtherButtonNode );
	pcView->SetRoot( pcRoot );
	AddChild( pcView );

	pcNewButton->SetTabOrder( os::NEXT_TAB_ORDER );
	pcEditButton->SetTabOrder( os::NEXT_TAB_ORDER );
	pcDeleteButton->SetTabOrder( os::NEXT_TAB_ORDER );
	pcOkButton->SetTabOrder( os::NEXT_TAB_ORDER );
	pcCancelButton->SetTabOrder( os::NEXT_TAB_ORDER );
	SetDefaultButton( pcOkButton );
	SetFocusChild( pcNewButton );
}

void UserInserts::LoadInserts()
{
	try
	{
		os::String zPath = getenv( "HOME" );
		zPath += "/Settings/Sourcery/Inserts";
		os::Settings * pcSettings = new os::Settings( new os::File( zPath, O_RDWR | O_CREAT ) );
		pcSettings->Load();

		for( int i = 0; i < pcSettings->CountNames(); i++ )
		{
			os::String cInsertText;
			os::String cName = pcSettings->GetName( i );
			pcSettings->FindString( cName.c_str(), &cInsertText );

			os::ListViewStringRow *pcRow = new os::ListViewStringRow();
			pcRow->AppendString( cName );
			pcRow->AppendString( cInsertText );
			pcInsertsList->InsertRow( pcRow );
		}
		delete pcSettings;
	}

	catch( ... )
	{
	}
}

void UserInserts::HandleMessage( os::Message * pcMessage )
{
	switch ( pcMessage->GetCode() )
	{
		case M_USER_INSERT_SAVE:
		{
			SaveInserts();
			OkToQuit();
			break;
		}
		case M_USER_INSERT_CANCEL:
		{
			OkToQuit();
			break;
		}
		case M_USER_INSERT_NEW:
		{
			UserInsertsEditWindow *pcNew = new UserInsertsEditWindow( this, true );
			pcNew->CenterInWindow( this );
			pcNew->Show();
			pcNew->MakeFocus();
			break;
		}
		case M_USER_INSERT_EDIT:
		{
			uint nSelected = pcInsertsList->GetLastSelected();
			if( nSelected >= 0 && pcInsertsList->GetRowCount() > 0 )
			{
				os::ListViewStringRow *pcRow = ( os::ListViewStringRow * ) pcInsertsList->GetRow( nSelected );
				os::String cName = pcRow->GetString( 0 );
				os::String cText = pcRow->GetString( 1 );
				UserInsertsEditWindow *pcEdit = new UserInsertsEditWindow( this, false );
				pcEdit->Insert( cName, cText );
				pcEdit->CenterInWindow( this );
				pcEdit->Show();
				pcEdit->MakeFocus();
			}
			break;
		}
		case M_USER_INSERT_DEL:
		{
			uint nSelected = pcInsertsList->GetLastSelected();
			if( nSelected >= 0 && pcInsertsList->GetRowCount() > 0 )
				pcInsertsList->RemoveRow( nSelected );
			break;
		}
		case M_USER_INSERT_EDIT_PASSED:
		{
			os::String cName, cText;
			pcMessage->FindString( "name", &cName );
			pcMessage->FindString( "text", &cText );
			uint nSelected = pcInsertsList->GetLastSelected();
			os::ListViewStringRow *pcRow = ( os::ListViewStringRow * ) pcInsertsList->GetRow( nSelected );
			pcRow->SetString( 0, cName );
			pcRow->SetString( 1, cText );
			break;
		}
		case M_USER_INSERT_NEW_PASSED:
		{
			os::String cName, cText;
			pcMessage->FindString( "name", &cName );
			pcMessage->FindString( "text", &cText );
			os::ListViewStringRow *pcRow = new os::ListViewStringRow();
			pcRow->AppendString( cName );
			pcRow->AppendString( cText );
			pcInsertsList->InsertRow( pcRow );
			break;
		}
		case M_USER_INSERT_MACROS:
		{
			os::String cMacros = "%C - Displays a comment\n%D - Displays the current date\n%F - Displays the current file name\n%N - Displays the name of the user(Change in Users Program)\n%S - Displays the system info\n%T - Displays the time";
			os::Alert *pcAlert = new os::Alert( "Insert Macros...", cMacros, LoadImageFromResource( "icon48x48.png" )->LockBitmap(), 0, "Okay", NULL );
			pcAlert->CenterInWindow( this );
			pcAlert->Go( new os::Invoker() );
			break;
		}
	}
}

void UserInserts::SaveInserts()
{
	try
	{
		os::String zPath = getenv( "HOME" );
		zPath += "/Settings/Sourcery/Inserts";
		os::Settings * pcSettings = new os::Settings( new os::File( zPath ) );

		for( int i = 0; i < pcSettings->CountNames(); i++ )
		{
			pcSettings->RemoveName( pcSettings->GetName( i ).c_str() );
		}

		for( uint i = 0; i < pcInsertsList->GetRowCount(); i++ )
		{
			os::ListViewStringRow *pcRow = ( os::ListViewStringRow * ) pcInsertsList->GetRow( i );
			pcSettings->SetString( pcRow->GetString( 0 ).c_str(), pcRow->GetString( 1 ) );
		}
		pcSettings->Save();
		delete( pcSettings );
	}
	catch( ... )
	{
	}
}

bool UserInserts::OkToQuit()
{
	pcParentWindow->PostMessage( new os::Message( M_USER_INSERT_QUIT ), pcParentWindow );
	return false;
}
